
# q-expansion of newform 252.3.g.b, downloaded from the LMFDB on 15 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 252
weight = 3
poly_data = [64, -96, 112, -88, 72, -44, 33, -22, 18, -11, 7, -3, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[2608, 784, 720, 916, 392, 551, 386, -66, 43, 89, 23, 31], 1264], [[-20832, 9968, -16848, 6500, -8604, 2401, -4482, 1734, -2207, 635, -475, 33], 1264], [[14688, -13552, 15904, -10868, 8708, -4649, 4162, -2606, 2191, -1403, 731, -265], 2528], [[-8240, -1120, -5904, -1444, -1982, -2243, -1206, -628, -637, -319, -157, -191], 1264], [[39968, -31248, 39920, -19716, 21032, -9231, 9534, -5134, 4877, -2193, 1521, -423], 2528], [[13824, 1744, 3072, 6668, 3084, 3083, 2458, 1330, 747, 1017, 135, 443], 2528], [[-27744, 28688, -31024, 24180, -18520, 10695, -8318, 6046, -4437, 2457, -1609, 799], 2528], [[39968, -51472, 53824, -42468, 26088, -18553, 12378, -10506, 6931, -4879, 2311, -897], 2528], [[-33248, 41488, -40112, 31564, -23812, 16173, -10290, 9070, -6163, 3823, -1959, 973], 2528], [[-1449, 1268, -1586, 1101, -946, 528, -389, 281, -213, 125, -68, 32], 79], [[12088, -18808, 15256, -15200, 9161, -7033, 3704, -3937, 2176, -1728, 734, -443], 632]]

hecke_ring_character_values = [[127, [-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [29, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [73, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, -1, 0, 0, 0, 0, 1, 0, 0, 0], [0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 3, 0, -1, 0, 0, -1, 0, 0, 0, -1, -1], [-3, 1, 1, 3, -1, 1, 1, -1, 0, 0, -1, -1], [3, 2, 1, 2, 3, -1, 3, -2, 3, 3, -3, -2], [1, -1, -1, -3, 1, 3, -1, -1, 0, 2, 1, 1], [-2, -6, 2, -3, -2, -4, 1, 1, 0, -1, -3, -4], [-6, -1, 0, -2, 4, -2, 0, 1, -2, 1, 0, 1], [0, 3, 0, 0, 0, -2, -4, -1, 0, -1, -4, -3], [-10, 1, -4, 0, 0, -2, 4, -1, 0, 3, -4, -1], [17, -2, -1, -6, -3, 1, 1, 2, -3, -3, -1, 2], [0, -2, 0, -4, 0, -8, -4, -6, 0, -2, -4, -6], [6, -2, -6, -4, 6, 6, -8, -4, 0, -2, 4, -2], [-10, 3, 8, 12, -4, 6, 0, -3, 0, -3, 0, -3], [1, 12, -1, 7, 1, 5, -3, -2, 0, 5, -1, 8], [5, 3, -3, 5, 11, -7, 5, -3, 2, 10, -5, -3], [2, 6, -2, 12, 2, -2, 0, -4, 0, -2, 4, 6], [-4, -6, 4, 13, -4, 14, 5, 11, 0, 11, -3, 16], [24, 6, 6, 16, -6, 6, 2, -6, 6, 0, -2, -6], [-8, -14, 8, -2, -8, -8, 6, 0, 0, 8, -10, 2], [1, -2, -1, -3, 1, -3, 3, 2, 0, -7, 5, -6], [51, -6, -11, -12, -1, -5, -13, 6, 1, -1, 13, 6], [-18, -2, 16, 2, 8, 4, -4, 2, -6, -6, 4, 2], [-6, 2, 10, 15, 6, 2, -2, -2, -5, 0, 2, -2], [0, 5, 0, 12, 0, 18, 0, 5, 0, 13, 0, 19], [-10, 17, 10, 5, -10, -30, 9, -6, 0, -4, -11, -7], [-16, 0, 2, -2, -10, 6, 2, 0, 0, -6, -2, 0], [-50, 6, -8, 12, 16, -12, 8, -6, 4, 18, -8, -6], [18, -40, -18, 0, 18, 6, -4, -10, 0, -20, 32, -4], [9, 4, -9, -39, 9, 17, -9, -12, 0, 11, 9, -4], [-20, -4, -2, -32, 10, -6, 10, 4, 6, 2, -10, 4], [-3, -6, 3, 45, -3, 9, 11, 18, 0, -3, 5, 22], [58, 3, 20, 0, 0, 10, 4, -3, 8, -7, -4, -3], [-6, -18, 6, 2, -6, -18, 6, 14, 0, -20, -6, -22], [-25, 1, -13, 39, -11, -1, -13, -1, -22, 2, 13, -1], [4, 46, -4, -30, 4, 0, 2, -8, 0, 0, 10, -10], [10, -6, -10, 4, 10, 10, -24, -4, 0, -6, -4, -6], [-32, -2, -4, 5, 12, -8, -8, 2, -5, 6, 8, 2], [-2, 9, 2, 19, -2, -10, 23, 0, 0, -6, 19, 13], [13, -1, -15, -37, 23, -19, 9, 1, 24, 18, -9, 1], [20, -44, -20, -43, 20, 10, -3, -31, 0, 1, 37, -2], [8, -9, 10, -34, -6, 8, -14, 9, 12, -17, 14, 9], [-34, -13, 8, -32, 12, -2, -24, 13, 4, -11, 24, 13], [-10, 8, 10, -30, -10, -26, 6, -20, 0, 14, -14, -8], [-22, 44, 22, 58, -22, -18, 14, 20, 0, 6, -30, 20], [12, -10, -12, -32, 12, 0, 0, -38, 0, 14, 24, 10], [-15, 56, 15, 3, -15, -27, -7, -2, 0, 5, -37, -12], [105, 9, -3, 51, -13, 5, -3, -9, -12, 4, 3, -9], [24, 4, 26, 24, 30, -2, -10, -4, 2, 6, 10, -4], [-14, -54, 14, 21, -14, -32, 1, -11, 0, 7, -27, 4], [62, 24, 8, 62, 24, -8, 12, -24, 22, 32, -12, -24], [-5, 22, 5, 5, -5, 3, 15, -4, 0, 17, 5, 22], [67, -10, -35, -40, 7, -21, 3, 10, -3, 11, -3, 10], [-20, -40, 20, 19, -20, -42, 11, -13, 0, 11, -29, 6], [-46, 4, -34, 37, -22, -6, -10, -4, -11, 10, 10, -4], [-10, -53, 10, -78, -10, -28, 18, -13, 0, 5, -2, -35], [-152, -19, -22, -34, -22, 0, 2, 19, -44, -19, -2, 19], [172, -14, 22, -44, -22, 22, -6, 14, -6, -36, 6, 14], [-23, -3, 23, -83, -23, 7, 23, 13, 0, 40, -23, -13], [46, -6, -10, -45, 26, -18, 14, 6, 7, 12, -14, 6], [25, -55, -25, 33, 25, -1, -37, -27, 0, -24, 13, -1], [-4, -38, 4, -62, -4, -12, -14, -16, 0, 12, -22, -30], [-36, 12, -18, 2, -22, 2, 22, -12, 24, 10, -22, -12], [-70, 33, -20, 68, -16, -2, 28, -33, 36, 35, -28, -33], [4, 24, -4, -20, 4, 0, -36, 8, 0, -16, -28, -40], [-154, 4, 20, 24, -4, 12, 28, -4, -36, -8, -28, -4], [16, 27, -16, -7, 16, 12, -39, 10, 0, -30, -7, -37], [167, -29, 19, -89, -19, 19, -29, 29, 2, -48, 29, 29], [213, 2, -9, -28, 21, -15, 9, -2, 27, 17, -9, -2], [-14, -4, 14, 75, -14, 8, 7, 41, 0, -5, -21, 22], [40, 52, -40, -80, 40, 72, -64, -28, 0, 20, 16, -12], [62, -8, -4, -44, 4, -4, -20, 8, 32, -4, 20, 8], [-32, 16, 32, -8, -32, 44, 32, 56, 0, 52, -32, 32], [2, -76, -2, 26, 2, -6, -42, -28, 0, 18, -38, 12], [90, -17, -16, -34, -12, -2, -24, 17, -10, -15, 24, 17], [155, -7, 31, 3, -7, 19, -1, 7, -30, -26, 1, 7], [-16, 12, 18, 56, 6, 6, 30, -12, -38, 6, -30, -12], [-86, 12, -52, 66, -12, -20, -24, -12, 6, 32, 24, -12], [-45, 42, 45, -23, -45, -37, 35, 4, 0, 49, -55, 10], [-32, 33, 18, -2, 34, -8, 58, -33, 76, 41, -58, -33], [30, -88, -30, -57, 30, 52, -21, 5, 0, -13, 39, -22], [-140, 16, -2, 22, 10, -6, 14, -16, 28, 22, -14, -16], [-48, -38, 48, 8, -48, -100, 32, -14, 0, 10, -64, -18], [-26, -85, 26, -153, -26, 10, 43, 18, 0, 44, -9, -37], [-66, -20, -8, -52, -40, 16, -16, 20, -12, -36, 16, 20], [-6, 8, -72, 64, 8, -40, 24, -8, -56, 48, -24, -8], [-40, -10, -4, -79, -4, 0, 36, 10, 3, -10, -36, 10], [4, -128, -4, 142, 4, -8, 6, 50, 0, -66, 14, 12], [-23, 126, 23, -73, -23, -55, 69, -24, 0, 15, 23, -10], [2, -28, -2, -60, 2, -10, 0, -74, 0, 60, 4, 32], [42, 14, -42, 46, 42, 134, -46, 58, 0, -8, 38, 34], [38, -31, -38, 79, 38, -18, -77, -52, 0, -42, -1, -3], [42, 30, -42, -94, 42, 58, -18, -42, 0, 16, 66, 2], [-4, -14, 4, 24, -4, -24, 24, 46, 0, -62, 16, -42], [-126, 12, -66, 5, -6, -30, 42, -12, 1, 42, -42, -12], [71, -14, 29, 22, -9, 19, -81, 14, 3, -33, 81, 14], [31, -56, -31, 15, 31, -33, 17, -24, 0, -71, 79, -24], [-316, 1, -2, -6, -58, 28, -26, -1, 36, -27, 26, -1], [-52, 20, 52, -2, -52, -108, 70, -46, 0, 42, -34, 24], [218, -9, -52, -60, 56, -54, -12, 9, 36, 45, 12, 9], [47, 200, -47, -23, 47, 67, -45, 14, 0, -41, 49, -28], [72, -30, -78, 20, -18, -30, -58, 30, -82, 0, 58, 30], [52, 156, -52, 2, 52, 44, -70, -50, 0, -10, 34, 8], [274, 0, 56, -44, -24, 40, 8, 0, 36, -40, -8, 0], [-41, 64, 41, 133, -41, -13, 15, 82, 0, -13, -67, 20], [-67, -14, 15, -68, -35, 25, -11, 14, 23, -39, 11, 14], [-12, 82, 12, -7, -12, 94, 33, 55, 0, 63, 9, 64], [242, -4, 56, 44, 24, 16, -8, 4, -52, -20, 8, 4], [-20, -62, 20, -188, -20, 0, 52, -22, 0, 62, 12, -26], [-116, -30, -50, 14, -46, -2, -98, 30, -36, -28, 98, 30], [-248, -14, 2, -20, 30, -14, -18, 14, -18, 0, 18, 14], [-29, 14, 29, 95, -29, -25, 33, -38, 0, 71, -25, 106], [84, 136, -84, 58, 84, 56, -126, -74, 0, -38, 42, 12], [-4, -14, -58, -44, 26, -42, 10, 14, -22, 28, -10, 14], [-5, 71, 5, 15, -5, 21, 45, 79, 0, -48, 35, -23], [-50, 66, 50, 108, -50, 38, 24, 64, 0, 74, -76, 90], [-6, 7, 16, -18, 4, 6, 0, -7, 46, 1, 0, -7], [36, 149, -36, 163, 36, 72, -53, 54, 0, -54, 19, 37], [-23, -27, -55, -111, 71, -63, 17, 27, -14, 36, -17, 27], [264, -35, -6, -26, 2, -4, -62, 35, -52, -31, 62, 35], [180, -30, 64, -27, -88, 76, -12, 30, -81, -106, 12, 30], [-6, -45, 6, -89, -6, 22, 67, -26, 0, 60, 55, 43], [-25, -12, 25, 155, -25, 15, -3, 64, 0, 1, -53, 52], [234, 11, 56, 42, 44, 6, 24, -11, -22, 5, -24, -11], [140, -48, -74, -174, 18, -46, -26, 48, 8, -2, 26, 48], [22, -76, -22, 274, 22, -10, -42, 32, 0, -86, 2, 52], [-28, 21, 28, -48, -28, -10, 60, -59, 0, 105, 4, 83], [-481, 33, 23, 85, 65, -21, -1, -33, 48, 54, 1, -33], [-4, -62, 4, -190, -4, 0, 18, 28, 0, -20, 10, -110], [-30, 192, 30, -181, -30, 64, 39, -27, 0, 151, -21, 50], [-10, 32, 10, -34, -10, 54, -94, 24, 0, 50, -114, -24], [592, 0, -46, -82, -26, -10, 2, 0, 80, 10, -2, 0], [-107, 2, -117, -50, -79, -19, 57, -2, 1, 21, -57, -2], [-256, 20, 66, 18, 102, -18, 58, -20, 4, 38, -58, -20], [-638, 54, 58, 115, 38, 10, 54, -54, 47, 44, -54, -54], [3, 98, -3, 31, 3, 55, -95, 78, 0, -29, -89, -58], [-272, 26, -4, 109, -36, 16, -24, -26, 19, 10, 24, -26], [-394, 42, 40, 176, 32, 4, 32, -42, -40, 38, -32, -42], [43, 20, -43, -105, 43, 123, -127, -8, 0, 45, -41, -28], [118, 27, 48, -64, 148, -50, 64, -27, 108, 77, -64, -27], [88, 26, -88, -174, 88, -40, -70, -172, 0, -44, 106, -78], [94, -69, -94, 157, 94, 198, -111, 40, 0, -30, 77, 87], [-389, -13, -37, -55, -123, 43, 67, 13, -64, -56, -67, 13], [94, 76, -94, 92, 94, 10, -96, -38, 0, -140, 92, -48], [-333, 67, -25, 163, 105, -65, 55, -67, 50, 132, -55, -67], [289, 14, -65, -126, 29, -47, 109, -14, 73, 61, -109, -14], [83, -185, -83, 19, 83, 109, -95, 35, 0, -92, 71, -47], [108, -8, -108, 19, 108, 126, -117, -21, 0, -69, 99, -10], [136, 34, 66, -94, 142, -38, 114, -34, 116, 72, -114, -34], [245, 34, -65, 40, 77, -71, 53, -34, 43, 105, -53, -34], [6, -190, -6, -194, 6, 62, 18, -30, 0, 80, 30, -2], [54, 94, -54, -184, 54, -66, -44, -128, 0, -46, 64, -106], [-18, 14, 18, -46, -18, 62, -42, 26, 0, 72, -78, 10], [-106, 76, 106, -169, -106, -60, 75, -23, 0, 175, -137, 22], [-32, 198, 32, 12, -32, -92, 36, -30, 0, 2, -28, -6], [-311, 30, -41, 82, -27, -7, 17, -30, 21, 37, -17, -30], [274, -34, -96, 24, -8, -44, -64, 34, -96, 10, 64, 34], [-824, -36, 48, -93, 16, 16, -20, 36, -31, -52, 20, 36], [-84, 119, 84, 79, -84, -188, -57, -16, 0, -4, -225, -77], [-214, 32, -28, -36, 92, -60, 44, -32, 120, 92, -44, -32], [62, 228, -62, -116, 62, -46, -112, -78, 0, -92, 12, -144], [-87, -54, 87, 33, -87, -43, -41, 82, 0, 49, -215, -42], [148, 24, 6, 16, -126, 66, 18, -24, 62, -42, -18, -24], [34, 150, -34, 166, 34, -42, 74, -70, 0, -40, 142, 114], [28, -114, -28, -280, 28, 72, 24, -134, 0, 150, 80, 50], [-79, -31, -15, -191, -113, 49, 1, 31, 66, -80, -1, 31]]
